//this file is part of eMule
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#include "stdafx.h"
#include "ED2KLink.h"
#include "otherfunctions.h"
#include "emule.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


namespace {
	struct autoFree {
		autoFree(TCHAR* p) : m_p(p) {}
		~autoFree() { free(m_p); }
	private:
		TCHAR * m_p;
	};
}

CED2KLink::~CED2KLink()
{
}

/////////////////////////////////////////////
// CED2KServerListLink implementation
/////////////////////////////////////////////
CED2KServerListLink::CED2KServerListLink(const CString &strAddress) : m_strAddress(strAddress)
{
}

CED2KServerListLink::~CED2KServerListLink()
{
}

void CED2KServerListLink::GetLink(CString& lnk)
{
	lnk.Format(_T("ed2k://|serverlist|%s|/"), m_strAddress);
}

CED2KServerListLink* CED2KServerListLink::GetServerListLink()
{
	return this;
}

CED2KServerLink* CED2KServerListLink::GetServerLink()
{
	return 0;
}

CED2KFileLink* CED2KServerListLink::GetFileLink()
{
	return 0;
}

CED2KLink::LinkType CED2KServerListLink::GetKind() const
{
	return kServerList;
}

/////////////////////////////////////////////
// CED2KServerLink implementation
/////////////////////////////////////////////
CED2KServerLink::CED2KServerLink(const CString &strIP, const TCHAR *port) : m_ip(strIP)
{
	unsigned long ul = _tcstoul(port,0,10);
	if ((ul > 0xFFFF) || (ul == 0))
		throw GetResString(IDS_ERR_BADPORT);
	m_port = static_cast<uint16>(ul);
	m_defaultName.Format(_T("Server %s:%s"), strIP, port);
}

CED2KServerLink::~CED2KServerLink()
{
}

void CED2KServerLink::GetLink(CString& lnk)
{
	lnk.Format(_T("ed2k://|server|%s|%u|/"), m_ip, m_port);
}

CED2KServerListLink* CED2KServerLink::GetServerListLink()
{
	return 0;
}

CED2KServerLink* CED2KServerLink::GetServerLink()
{
	return this;
}

CED2KFileLink* CED2KServerLink::GetFileLink()
{
	return 0;
}

CED2KLink::LinkType CED2KServerLink::GetKind() const
{
	return kServer;
}


/////////////////////////////////////////////
// CED2KFileLink implementation
/////////////////////////////////////////////
CED2KFileLink::CED2KFileLink(const CString &strName, const TCHAR* size, const CString &strHash, const TCHAR* sources)
: m_name(strName)
{
	uint64	qwSz = _tstoi64(size);
	if (qwSz >= static_cast<uint64>(4294967295))
		throw GetResString(IDS_ERR_TOOLARGEFILE);

	m_dwSize = static_cast<uint32>(qwSz);

	if (md4cmp0(StringToHash(strHash, m_hash)) == 0)
		throw GetResString(IDS_ERR_ILLFORMEDHASH);

	m_name.Trim();
	if (m_name.IsEmpty())
		throw GetResString(IDS_ERR_LINKERROR);

	ClientSourcesList=NULL; //<<-- enkeyDEV(kei-kun) -hostname sources in ED2K superLink-

	if (sources)
	{
		TCHAR* pNewString = _tcsdup(sources);
		autoFree liberator(pNewString);
		TCHAR* pCh = pNewString;
		TCHAR* pEnd;
		TCHAR* pIP;
		TCHAR* pPort;

		bool bAllowSources;
		TCHAR date[3];
		COleDateTime expirationDate;
		int nYear,nMonth,nDay;

		uint16 nHostnameCount = 0; //<<-- enkeyDEV(kei-kun) -hostname sources in ED2K superLink-
		uint16 nCount = 0;
		uint32 dwID;
		uint16 nPort;
		uint32 dwServerIP = 0;
		uint16 nServerPort = 0;
		unsigned long ul;

		int nInvalid = 0;

		pCh = _tcsstr( pCh, _T("sources") );
		if( pCh != NULL )
		{
			pCh = pCh + 7; // point to char after "sources"
			pEnd = pCh;
			while( *pEnd )
			{
			//	Make pEnd point to the terminating NULL
				pEnd++;
			}
			bAllowSources=true;
			// if there's an expiration date...
			if( *pCh == _T('@') && (pEnd-pCh) > 7 )
			{
				pCh++; // after '@'
				date[2] = 0; // terminate the two character string
				date[0] = *(pCh++); date[1] = *(pCh++);
				nYear = _tcstol( date, 0, 10 ) + 2000;
				date[0] = *(pCh++); date[1] = *(pCh++);
				nMonth = _tcstol( date, 0, 10 );
				date[0] = *(pCh++); date[1] = *(pCh++);
				nDay = _tcstol( date, 0, 10 );
				bAllowSources = ( expirationDate.SetDate(nYear,nMonth,nDay) == 0 );
				if (bAllowSources)
					bAllowSources=(COleDateTime::GetCurrentTime() < expirationDate);
			}

			// increment pCh to point to the first "ip:port" or "address" and check for sources
			if ( bAllowSources && ++pCh < pEnd )
			{
				ClientSourcesList=new CTypedPtrList<CPtrList, CClientSource*>(); //<<-- enkeyDEV(kei-kun) -hostname sources in ED2K superLink-
				// for each "ip:port"/"name:port" source string until the end
				// limit to prevent overflow (uint16 due to CPartFile::AddClientSources)
				while( *pCh != 0 && nCount < MAXSHORT )
				{
					pIP = pCh;
					// find the end of this ip:port string & start of next ip:port string.
					if( pCh = _tcschr(pCh, _T(',')) )
					{
						*pCh = 0; // terminate current "ip:port"/"name:port"
						pCh++;    // point to next "ip:port"/"name:port"
					}
					else
						pCh = pEnd;

					// if port is not present for this ip/name, go to the next ip/name.
					if( (pPort = _tcschr(pIP, _T(':'))) == NULL )
					{
						nInvalid++;	continue;
					}

					*pPort = 0;     // terminate ip/name string
					pPort++;	// point pPort to port string.

					dwID = inet_addr( pIP );
					// START enkeyDEV(kei-kun) -hostname sources in ED2K superLink-
					ul = _tcstoul( pPort, 0, 10 );
					nPort = static_cast<uint16>(ul);

					// skip bad ports
					if (ul > 0xFFFF || ul == 0)
					{
						nInvalid++;
						continue;
					}

					CClientSource* newsource = new CClientSource;
				//	could it be a hostname? We will assume this for later checking...
					if (dwID == INADDR_NONE)
					{
						if (_tcslen(pIP) > 512 /*hostsource->hostname*/)
						{
							nInvalid++;
							delete newsource;
							continue;
						}
						newsource->sourceType=ED2KLINK_SOURCE_HOSTNAME;
						_tcscpy(newsource->sourceHostname, pIP);   //save hostname
						nHostnameCount++;
					}
					else
					{
					//	skip bad ips
						if(dwID < 0x1000000)
						{
							nInvalid++;
							delete newsource;
							continue;
						}
						newsource->sourceType=ED2KLINK_SOURCE_IP;
						newsource->sourceID=dwID; //save ID
					}
					newsource->sourcePort=nPort; //save port
					md4cpy(newsource->filehashkey, m_hash); //save file hash

					newsource->serverIP=dwServerIP;
					newsource->serverPort=nServerPort;
					ClientSourcesList->AddTail(newsource);
					nCount++;
				}
				if (nCount==0 && nHostnameCount==0)
				{
					delete ClientSourcesList;
					ClientSourcesList=NULL;
				}
				// END enkeyDEV(kei-kun) -hostname sources in ED2K superLink-
			}
		}
	}
}


CED2KFileLink::~CED2KFileLink()
{
	if (ClientSourcesList)
		delete ClientSourcesList;
}

void CED2KFileLink::GetLink(CString& lnk)
{
	TCHAR	acHashStr[MAX_HASHSTR_SIZE];

	lnk.Format(_T("ed2k://|file|%s|%u|%s|/"), m_name, m_dwSize, md4str(m_hash, acHashStr));
}

CED2KServerListLink* CED2KFileLink::GetServerListLink()
{
	return 0;
}

CED2KServerLink* CED2KFileLink::GetServerLink()
{
	return 0;
}
CED2KFileLink* CED2KFileLink::GetFileLink()
{
	m_name.Replace(_T("%20"),_T(" "));
	return this;
}

CED2KLink::LinkType CED2KFileLink::GetKind() const
{
	return kFile;
}

CED2KLink* CED2KLink::CreateLinkFromUrl(const CString &strURI)
{
	int iPos = 0;
	CString strTok = strURI.Tokenize(_T("|"), iPos);
	if (strTok == _T("ed2k://"))
	{
		strTok = strURI.Tokenize(_T("|"), iPos);
		if (strTok == _T("file"))
		{
			CString	strName = strURI.Tokenize(_T("|"), iPos);
			if (!strName.IsEmpty())
			{
				CString	strSize = strURI.Tokenize(_T("|"), iPos);
				if (!strSize.IsEmpty())
				{
					CString	strHash = strURI.Tokenize(_T("|"), iPos);
					if (!strHash.IsEmpty())
					{
						CStringArray astrEd2kParams;
						bool bEmuleExt = false, bLastTokSlash = false;
						CString	strEmuleExt;

						for (;;)
						{
							strTok = strURI.Tokenize(_T("|"), iPos);
							if (strTok.IsEmpty())
								break;

							if (strTok == _T("/"))
							{
								bLastTokSlash = true;
								if (bEmuleExt)
									break;
								bEmuleExt = true;
							}
							else
							{
								bLastTokSlash = false;
								if (bEmuleExt)
								{
									if (!strEmuleExt.IsEmpty())
										strEmuleExt += _T('|');
									strEmuleExt += strTok;
								}
								else
									astrEd2kParams.Add(strTok);
							}
						}

						if (bLastTokSlash)
							return new CED2KFileLink(strName, strSize, strHash, strEmuleExt.IsEmpty() ? (LPCTSTR)NULL : (LPCTSTR)strEmuleExt);
					}
				}
			}
		}
		else if (strTok == _T("serverlist"))
		{
			strTok = strURI.Tokenize(_T("|"), iPos);	//	URL
			if (!strTok.IsEmpty() && strURI.Tokenize(_T("|"), iPos) == _T("/"))
				return new CED2KServerListLink(strTok);
		}
		else if (strTok == _T("server"))
		{
			strTok = strURI.Tokenize(_T("|"), iPos);	//	Server
			if (!strTok.IsEmpty())
			{
				CString	strPort = strURI.Tokenize(_T("|"), iPos);
				if (!strPort.IsEmpty() && strURI.Tokenize(_T("|"), iPos) == _T("/"))
					return new CED2KServerLink(strTok, strPort);
			}
		}
	}

	throw GetResString(IDS_ERR_LINKERROR);
	return NULL;
}
